'use strict';

// markdown can have markdown [test](url) links, or bare URLs.  Anything else will be ignored and remain text.  Only link entities are set; that's all that's needed to post to the app.net API.
exports.parse = function(markdown) {
  // Markdown bracket regex based on http://stackoverflow.com/a/9268827
  var markdownLinkRegex = /\[([^\]]+)\]\((\S+(?=\)))\)/;

  // Regex pulled from https://github.com/chriso/node-validator and country codes pulled from http://data.iana.org/TLD/tlds-alpha-by-domain.txt
  var bareUrlRegex = /((?:http|https|ftp|scp|sftp):\/\/)?(?:\w+:\w+@)?(?:localhost|(?:(?:[\-\w\d{1-3}]+\.)+(?:com|org|net|gov|mil|biz|info|mobi|name|aero|jobs|edu|co\.uk|ac\.uk|it|fr|tv|museum|asia|local|travel|int|AC|AD|AE|AF|AG|AI|AL|AM|AN|AO|AQ|AR|AS|AT|AU|AW|AX|AZ|BA|BB|BD|BE|BF|BG|BH|BI|BJ|BM|BN|BO|BR|BS|BT|BV|BW|BY|BZ|CA|CC|CD|CF|CG|CH|CI|CK|CL|CM|CN|CO|CR|CU|CV|CW|CX|CY|CZ|DE|DJ|DK|DM|DO|DZ|EC|EE|EG|ER|ES|ET|EU|FI|FJ|FK|FM|FO|FR|GA|GB|GD|GE|GF|GG|GH|GI|GL|GM|GN|GP|GQ|GR|GS|GT|GU|GW|GY|HK|HM|HN|HR|HT|HU|ID|IE|IL|IM|IN|IO|IQ|IR|IS|IT|JE|JM|JO|JP|KE|KG|KH|KI|KM|KN|KP|KR|KW|KY|KZ|LA|LB|LC|LI|LK|LR|LS|LT|LU|LV|LY|MA|MC|MD|ME|MG|MH|MK|ML|MM|MN|MO|MP|MQ|MR|MS|MT|MU|MV|MW|MX|MY|MZ|NA|NC|NE|NF|NG|NI|NL|NO|NP|NR|NU|NZ|OM|PA|PE|PF|PG|PH|PK|PL|PM|PN|PR|PS|PT|PW|PY|QA|RE|RO|RS|RU|RW|SA|SB|SC|SD|SE|SG|SH|SI|SJ|SK|SL|SM|SN|SO|SR|ST|SU|SV|SX|SY|SZ|TC|TD|TF|TG|TH|TJ|TK|TL|TM|TN|TO|TP|TR|TT|TV|TW|TZ|UA|UG|UK|US|UY|UZ|VA|VC|VE|VG|VI|VN|VU|WF|WS|YE|YT|ZA|ZM|ZW))|(?:(?:\b25[0-5]\b|\b[2][0-4][0-9]\b|\b[0-1]?[0-9]?[0-9]\b)(?:\.(?:\b25[0-5]\b|\b[2][0-4][0-9]\b|\b[0-1]?[0-9]?[0-9]\b)){3}))(?::[\d]{1,5})?(?:(?:(?:\/(?:[\-\w~!$+|.,="'\(\)_\*:]|%[a-f\d]{2})+)+|\/)+|\?|#)?(?:(?:\?(?:[\-\w~!$+|.,*:]|%[a-f\d{2}])+=?(?:[\-\w~!$+|.,*:=]|%[a-f\d]{2})*)(?:&(?:[\-\w~!$+|.,*:]|%[a-f\d{2}])+=?(?:[\-\w~!$+|.,*:=]|%[a-f\d]{2})*)*)*(?:#(?:[\-\w~!$ |\/.,*:;=]|%[a-f\d]{2})*)?/ig;

  var links = [];
  var text = markdown;

  function handleReplacement(_, anchor, url, pos) {
    links.push({
      pos: pos,
      len: anchor.length,
      url: url
    });

    return anchor;
  }

  var oldText;

  // Has to be called repeatedly, since if done globally, it will provide the original index (before earlier replacements)
  do {
    oldText = text;
    text = oldText.replace(markdownLinkRegex, handleReplacement);
  } while(text !== oldText);

  var match;
  while((match = bareUrlRegex.exec(text))) {
    var url = match[0];
    var len = url.length;

    if(match[1] === undefined) {
      url = 'http://' + url;
    }

    links.push({
      pos: match.index,
      len: len,
      url: url
    });
  }

  return {
    text: text,
    entities: {
      links: links
    }
  };
};

// Return markdown given app.net object, generally either a post or user description.  Only text and entities.links are currently used
exports.stringify = function(adnObject) {
  var markdown = adnObject.text;
  var links = adnObject.entities.links;
  var link, len, remaining;
  links.sort(function(l1, l2) {
    return l1.pos - l2.pos;
  });

  for(var i = links.length - 1; i >= 0; i--) {
    link = links[i];

    // We keep bare URLs bare.
    if(link.text !== link.url) {
      len = link.amended_len || link.len; // amended_len is omitted when there is no anti-phishing protection
      remaining = markdown.substring(link.pos + len);
      markdown = markdown.substring(0, link.pos) + '[' + link.text + '](' + link.url + ')' + remaining;
    }
  }

  return markdown;
};
